const path = require('path')
const webpack = require('webpack')
const ExtractTextPlugin = require('extract-text-webpack-plugin')
const NODE_ENV = process.env.NODE_ENV || 'development'

const __DEV__ = NODE_ENV === 'development'
const SRC_PATH = path.resolve(__dirname, './src')
const DIST_PATH = path.resolve(__dirname, './dist')
const STYLE_PATH = path.resolve(__dirname, './src/styles')

const config = {
  context: SRC_PATH,
  entry: './index.js',
  devtool: false,
  output: {
    path: DIST_PATH,
    filename: 'react-web-ui.js',
    library: {
      root: 'ReactWebUI',
      amd: 'react-web-ui',
      commonjs: 'react-web-ui'
    },
    libraryTarget: 'umd'
  },
  externals: {
    'immutable': {
      commonjs: 'immutable',
      commonjs2: 'immutable',
      amd: 'immutable',
      root: 'Immutable'
    },
    'react': {
      commonjs: 'react',
      commonjs2: 'react',
      amd: 'react',
      root: 'React'
    },
    'react-dom': {
      commonjs: 'react-dom',
      commonjs2: 'react-dom',
      amd: 'react-dom',
      root: 'ReactDom'
    },
    'prop-types': {
      commonjs: 'prop-types',
      commonjs2: 'prop-types',
      amd: 'prop-types',
      root: 'PropTypes'
    }
  },
  resolve: {
    modules: [path.resolve(__dirname, './node_modules')],
    extensions: ['.js', '.jsx', '.json']
  },
  module: {
    rules: []
  },
  plugins: [
    new webpack.DefinePlugin(Object.assign({
      'process.env': {
        NODE_ENV: JSON.stringify(NODE_ENV)
      }
    }))
  ],
  // 精确控制要显示的 bundle 信息
  stats: {
    assets: true,
    colors: true,
    errors: true,
    errorDetails: true,
    hash: true
  }
}

// ESlint
config.module.rules.push({
  test: /\.jsx?$/,
  enforce: 'pre',
  exclude: /node_modules/,
  include: SRC_PATH,
  loader: 'eslint-loader'
})

// JavaScript
config.module.rules.push({
  test: /\.jsx?$/,
  include: SRC_PATH,
  exclude: /node_modules/,
  loader: 'babel-loader'
})

// Styles
const cssLoader = {
  loader: 'css-loader',
  options: {
    modules: true,
    camelCase: true,
    importLoaders: 1,
    localIdentName: '[name]-[local]-[hash:base64:4]',
    minimize: {
      autoprefixer: {
        add: true,
        remove: true
      },
      discardComments: {
        removeAll: true
      },
      discardUnused: false,
      mergeIdents: false,
      reduceIdents: false,
      safe: true,
      uniqueSelectors: true,
      sourcemap: __DEV__
    }
  }
}

const sassLoader = {
  loader: 'sass-loader',
  options: {
    sourceMap: __DEV__,
    includePaths: [STYLE_PATH],
    outputStyle: 'expanded'
  }
}

config.module.rules.push({
  test: /\.s?css$/,
  use: ExtractTextPlugin.extract({
    fallback: 'style-loader',
    use: [cssLoader, sassLoader]
  })
})

config.plugins.push(
  new ExtractTextPlugin({
    filename: 'react-web-ui.css',
    allChunks: true,
    disable: !__DEV__
  })
)

// Images
// ------------------------------------
config.module.rules.push({
  test: /\.(png|jpg|gif)$/,
  loader: 'url-loader',
  options: { limit: 8192 }
})


// Fonts
;[
  ['woff', 'application/font-woff'],
  ['woff2', 'application/font-woff2'],
  ['otf', 'font/opentype'],
  ['ttf', 'application/octet-stream'],
  ['eot', 'application/vnd.ms-fontobject'],
  ['svg', 'image/svg+xml']
].forEach((font) => {
  const extension = font[0]
  const mimetype = font[1]

  config.module.rules.push({
    test: new RegExp(`\\.${extension}$`),
    loader: 'url-loader',
    options: {
      name: 'fonts/[name].[ext]',
      limit: 10000,
      mimetype
    }
  })
})

config.plugins.push(
  new webpack.LoaderOptionsPlugin({ minimize: true, debug: false }),
  new webpack.optimize.UglifyJsPlugin({
    sourceMap: !!config.devtool,
    comments: false,

    // 是否美化输出
    beautify: true,

    // 是否改变变量名
    mangle: false,
    compress: {
      warnings: false,
      screw_ie8: true,
      conditionals: true,
      unused: true,
      comparisons: true,
      sequences: true,
      dead_code: true,
      evaluate: true,
      if_return: true,
      join_vars: true
    }
  })
)

module.exports = config
